# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Compute Log-likelihood Value for RBM
#'
#' The two functions compute the exact and approximate log-likelihood values for RBM, respectively.
#'
#' @param w       Weight parameter of the RBM, of size \code{[m x n]}.
#' @param b       Bias parameter for the visible units, of size \code{[m x 1]}.
#' @param c       Bias parameter for the hidden units, of size \code{[n x 1]}.
#' @param dat     The observed data, of size \code{[m x N]}.
#' @param nsamp   Size of the Monte Carlo sample for approximation.
#' @param nstep   Number of steps in the Gibbs sampler.
#' @param vr      Whether to use variance reduction technique.
#' @param nthread Number of threads for parallel computing, if OpenMP is supported.
#'
#' @examples
#' set.seed(123)
#' m = 10
#' n = 10
#' b = rnorm(m, sd = 0.1)
#' c = rnorm(n, sd = 0.1)
#' w = matrix(rnorm(m * n, sd = 1.0), m, n)
#'
#' N = 100
#' dat = matrix(0, m, N)
#' v0 = rbinom(m, 1, 0.5)
#' for(i in 1:N)
#' {
#'     dat[, i] = rbm_sample_k(w, b, c, v0, k = 100)$v
#' }
#'
#' loglik_rbm(w, b, c, dat)
#' loglik_rbm_approx(w, b, c, dat, nsamp = 100, nstep = 10)
#' loglik_rbm_approx(w, b, c, dat, nsamp = 100, nstep = 100)
#'
#' @rdname loglik_rbm
#'
loglik_rbm <- function(w, b, c, dat) {
    .Call(`_cdtau_loglik_rbm`, w, b, c, dat)
}

#' @rdname loglik_rbm
loglik_rbm_approx <- function(w, b, c, dat, nsamp = 100L, nstep = 10L, vr = TRUE, nthread = 1L) {
    .Call(`_cdtau_loglik_rbm_approx`, w, b, c, dat, nsamp, nstep, vr, nthread)
}

#' Sample from an RBM Model
#'
#' \code{rbm_sample_k()} uses Gibbs sampling with a fixed-size chain.
#' \code{rbm_sample_tau()} uses the unbiased Gibbs sampling with a chain path.
#'
#' @param w         Weight parameter of the RBM, of size \code{[m x n]}.
#' @param b         Bias parameter for the visible units, of size \code{[m x 1]}.
#' @param c         Bias parameter for the hidden units, of size \code{[n x 1]}.
#' @param v0        The initial value for Gibbs sampling, of size \code{[m x 1]}.
#' @param k         Number of steps in the Gibbs sampler.
#' @param min_steps Minimum number of steps in the Gibbs sampler.
#' @param max_steps Maximum number of steps in the Gibbs sampler.
#' @param verbose   Whether to print algorithmic information.
#'
#' @examples
#' set.seed(123)
#' m = 10
#' n = 10
#' b = rnorm(m, sd = 0.1)
#' c = rnorm(n, sd = 0.1)
#' w = matrix(rnorm(m * n, sd = 1.0), m, n)
#' v0 = rbinom(m, 1, 0.5)
#'
#' rbm_sample_k(w, b, c, v0, k = 100)
#' rbm_sample_tau(w, b, c, v0, min_steps = 1, max_steps = 100, verbose = TRUE)
#'
#' @rdname rbm_sample
#'
rbm_sample_k <- function(w, b, c, v0, k = 10L) {
    .Call(`_cdtau_rbm_sample_k`, w, b, c, v0, k)
}

#' @rdname rbm_sample
rbm_sample_tau <- function(w, b, c, v0, min_steps = 10L, max_steps = 100L, verbose = FALSE) {
    .Call(`_cdtau_rbm_sample_tau`, w, b, c, v0, min_steps, max_steps, verbose)
}

rbm_cdk_warm_ <- function(vis_dim, hid_dim, dat, b0, c0, w0, batch_size = 10L, lr = 0.1, momentum = 0.0, niter = 100L, ngibbs = 10L, nchain = 1L, persistent = FALSE, eval_loglik = FALSE, exact_loglik = FALSE, eval_freq = 10L, eval_size = 100L, eval_nmc = 100L, eval_nstep = 10L, nthread = 1L, verbose = 0L) {
    .Call(`_cdtau_rbm_cdk_warm_`, vis_dim, hid_dim, dat, b0, c0, w0, batch_size, lr, momentum, niter, ngibbs, nchain, persistent, eval_loglik, exact_loglik, eval_freq, eval_size, eval_nmc, eval_nstep, nthread, verbose)
}

rbm_cdk_ <- function(vis_dim, hid_dim, dat, batch_size = 10L, lr = 0.1, momentum = 0.0, niter = 100L, ngibbs = 10L, nchain = 1L, persistent = FALSE, eval_loglik = FALSE, exact_loglik = FALSE, eval_freq = 10L, eval_size = 100L, eval_nmc = 100L, eval_nstep = 10L, nthread = 1L, verbose = 0L) {
    .Call(`_cdtau_rbm_cdk_`, vis_dim, hid_dim, dat, batch_size, lr, momentum, niter, ngibbs, nchain, persistent, eval_loglik, exact_loglik, eval_freq, eval_size, eval_nmc, eval_nstep, nthread, verbose)
}

rbm_ucd_warm_ <- function(vis_dim, hid_dim, dat, b0, c0, w0, batch_size = 10L, lr = 0.1, momentum = 0.0, niter = 100L, min_mcmc = 1L, max_mcmc = 100L, nchain = 1L, eval_loglik = FALSE, exact_loglik = FALSE, eval_freq = 10L, eval_size = 100L, eval_nmc = 100L, eval_nstep = 10L, nthread = 1L, verbose = 0L) {
    .Call(`_cdtau_rbm_ucd_warm_`, vis_dim, hid_dim, dat, b0, c0, w0, batch_size, lr, momentum, niter, min_mcmc, max_mcmc, nchain, eval_loglik, exact_loglik, eval_freq, eval_size, eval_nmc, eval_nstep, nthread, verbose)
}

rbm_ucd_ <- function(vis_dim, hid_dim, dat, batch_size = 10L, lr = 0.1, momentum = 0.0, niter = 100L, min_mcmc = 1L, max_mcmc = 100L, nchain = 1L, eval_loglik = FALSE, exact_loglik = FALSE, eval_freq = 10L, eval_size = 100L, eval_nmc = 100L, eval_nstep = 10L, nthread = 1L, verbose = 0L) {
    .Call(`_cdtau_rbm_ucd_`, vis_dim, hid_dim, dat, batch_size, lr, momentum, niter, min_mcmc, max_mcmc, nchain, eval_loglik, exact_loglik, eval_freq, eval_size, eval_nmc, eval_nstep, nthread, verbose)
}

